/*******************************************************************************
* File Name: Clk4X.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Clk4X.h"

static Clk4X_BACKUP_STRUCT  Clk4X_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: Clk4X_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet Clk4X_SUT.c usage_Clk4X_Sleep_Wakeup
*******************************************************************************/
void Clk4X_Sleep(void)
{
    #if defined(Clk4X__PC)
        Clk4X_backup.pcState = Clk4X_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            Clk4X_backup.usbState = Clk4X_CR1_REG;
            Clk4X_USB_POWER_REG |= Clk4X_USBIO_ENTER_SLEEP;
            Clk4X_CR1_REG &= Clk4X_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Clk4X__SIO)
        Clk4X_backup.sioState = Clk4X_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        Clk4X_SIO_REG &= (uint32)(~Clk4X_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: Clk4X_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to Clk4X_Sleep() for an example usage.
*******************************************************************************/
void Clk4X_Wakeup(void)
{
    #if defined(Clk4X__PC)
        Clk4X_PC = Clk4X_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            Clk4X_USB_POWER_REG &= Clk4X_USBIO_EXIT_SLEEP_PH1;
            Clk4X_CR1_REG = Clk4X_backup.usbState;
            Clk4X_USB_POWER_REG &= Clk4X_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Clk4X__SIO)
        Clk4X_SIO_REG = Clk4X_backup.sioState;
    #endif
}


/* [] END OF FILE */
