/*******************************************************************************
* File Name: PA_drv.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "PA_drv.h"

static PA_drv_BACKUP_STRUCT  PA_drv_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: PA_drv_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet PA_drv_SUT.c usage_PA_drv_Sleep_Wakeup
*******************************************************************************/
void PA_drv_Sleep(void)
{
    #if defined(PA_drv__PC)
        PA_drv_backup.pcState = PA_drv_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            PA_drv_backup.usbState = PA_drv_CR1_REG;
            PA_drv_USB_POWER_REG |= PA_drv_USBIO_ENTER_SLEEP;
            PA_drv_CR1_REG &= PA_drv_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PA_drv__SIO)
        PA_drv_backup.sioState = PA_drv_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        PA_drv_SIO_REG &= (uint32)(~PA_drv_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: PA_drv_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to PA_drv_Sleep() for an example usage.
*******************************************************************************/
void PA_drv_Wakeup(void)
{
    #if defined(PA_drv__PC)
        PA_drv_PC = PA_drv_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            PA_drv_USB_POWER_REG &= PA_drv_USBIO_EXIT_SLEEP_PH1;
            PA_drv_CR1_REG = PA_drv_backup.usbState;
            PA_drv_USB_POWER_REG &= PA_drv_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PA_drv__SIO)
        PA_drv_SIO_REG = PA_drv_backup.sioState;
    #endif
}


/* [] END OF FILE */
