/*******************************************************************************
* File Name: SignalBuffer.c
* Version 1.20
*
* Description:
*  This file provides the source code to the API for the Opamp (Analog Buffer)
*  Component.
*
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SignalBuffer.h"

uint8 SignalBuffer_initVar = 0u; /* Defines if component was initialized */
static uint32 SignalBuffer_internalPower = 0u; /* Defines component Power value */


/*******************************************************************************
* Function Name: SignalBuffer_Init
********************************************************************************
*
* Summary:
*  Initializes or restores the component according to the customizer Configure 
*  dialog settings. It is not necessary to call Init() because the Start() API 
*  calls this function and is the preferred method to begin the component operation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SignalBuffer_Init(void)
{
    SignalBuffer_internalPower = SignalBuffer_POWER;
    SignalBuffer_CTB_CTRL_REG = SignalBuffer_DEFAULT_CTB_CTRL;
    SignalBuffer_OA_RES_CTRL_REG = SignalBuffer_DEFAULT_OA_RES_CTRL;
    SignalBuffer_OA_COMP_TRIM_REG = SignalBuffer_DEFAULT_OA_COMP_TRIM_REG;
}


/*******************************************************************************
* Function Name: SignalBuffer_Enable
********************************************************************************
*
* Summary:
*  Activates the hardware and begins the component operation. It is not necessary to 
*  call Enable() because the Start() API calls this function, which is the 
*  preferred method to begin the component operation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SignalBuffer_Enable(void)
{
    SignalBuffer_OA_RES_CTRL_REG |= SignalBuffer_internalPower | \
                                        SignalBuffer_OA_PUMP_EN;
}


/*******************************************************************************
* Function Name: SignalBuffer_Start
********************************************************************************
*
* Summary:
*  Performs all of the required initialization for the component and enables power 
*  to the block. The first time the routine is executed, the Power level, Mode, 
*  and Output mode are set. When called to restart the Opamp following a Stop() call, 
*  the current component parameter settings are retained.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  SignalBuffer_initVar: Used to check the initial configuration, modified
*  when this function is called for the first time.
*
*******************************************************************************/
void SignalBuffer_Start(void)
{
    if( 0u == SignalBuffer_initVar)
    {
        SignalBuffer_Init();
        SignalBuffer_initVar = 1u;
    }
    SignalBuffer_Enable();
}


/*******************************************************************************
* Function Name: SignalBuffer_Stop
********************************************************************************
*
* Summary:
*  Turn off the Opamp block.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SignalBuffer_Stop(void)
{
    SignalBuffer_OA_RES_CTRL_REG &= ((uint32)~(SignalBuffer_OA_PWR_MODE_MASK | \
                                                   SignalBuffer_OA_PUMP_EN));
}


/*******************************************************************************
* Function Name: SignalBuffer_SetPower
********************************************************************************
*
* Summary:
*  Sets the Opamp to one of the three power levels.
*
* Parameters:
*  power: power levels.
*   SignalBuffer_LOW_POWER - Lowest active power
*   SignalBuffer_MED_POWER - Medium power
*   SignalBuffer_HIGH_POWER - Highest active power
*
* Return:
*  None
*
**********************************************************************************/
void SignalBuffer_SetPower(uint32 power)
{
    uint32 tmp;
    
    SignalBuffer_internalPower = SignalBuffer_GET_OA_PWR_MODE(power);
    tmp = SignalBuffer_OA_RES_CTRL_REG & \
           (uint32)~SignalBuffer_OA_PWR_MODE_MASK;
    SignalBuffer_OA_RES_CTRL_REG = tmp | SignalBuffer_internalPower;
}


/*******************************************************************************
* Function Name: SignalBuffer_PumpControl
********************************************************************************
*
* Summary:
*  Allows the user to turn the Opamp's boost pump on or off. By Default the Start() 
*  function turns on the pump. Use this API to turn it off. The boost must be 
*  turned on when the supply is less than 2.7 volts and off if the supply is more 
*  than 4 volts.
*
* Parameters:
*  onOff: Control the pump.
*   SignalBuffer_PUMP_OFF - Turn off the pump
*   SignalBuffer_PUMP_ON - Turn on the pump
*
* Return:
*  None
*
**********************************************************************************/
void SignalBuffer_PumpControl(uint32 onOff)
{
    
    if(0u != onOff)
    {
        SignalBuffer_OA_RES_CTRL |= SignalBuffer_OA_PUMP_EN;    
    }
    else
    {
        SignalBuffer_OA_RES_CTRL &= (uint32)~SignalBuffer_OA_PUMP_EN;
    }
}


/* [] END OF FILE */
