/*******************************************************************************
* File Name: SignalBuffer.h
* Version 1.20
*
* Description:
*  This file contains the function prototypes and constants used in
*  the Opamp (Analog Buffer) Component.
*
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/


#if !defined(CY_OPAMP_SignalBuffer_H)
#define CY_OPAMP_SignalBuffer_H

#include "cytypes.h"
#include "cyfitter.h"


/***************************************
*       Type Definitions
***************************************/

/* Structure to save state before go to sleep */
typedef struct
{
    uint8  enableState;
} SignalBuffer_BACKUP_STRUCT;


/**************************************
*        Function Prototypes
**************************************/
void SignalBuffer_Init(void);
void SignalBuffer_Enable(void);
void SignalBuffer_Start(void);
void SignalBuffer_Stop(void);
void SignalBuffer_SetPower(uint32 power);
void SignalBuffer_PumpControl(uint32 onOff);
void SignalBuffer_Sleep(void);
void SignalBuffer_Wakeup(void);
void SignalBuffer_SaveConfig(void);
void SignalBuffer_RestoreConfig(void);


/**************************************
*           API Constants
**************************************/

/* Parameters for SetPower() function */
#define SignalBuffer_LOW_POWER      (1u)
#define SignalBuffer_MED_POWER      (2u)
#define SignalBuffer_HIGH_POWER     (3u)


/* Parameters for PumpControl() function */
#define SignalBuffer_PUMP_ON        (1u)
#define SignalBuffer_PUMP_OFF       (0u)


/***************************************
*   Initial Parameter Constants
****************************************/

#define SignalBuffer_OUTPUT_CURRENT         (1u)
#define SignalBuffer_POWER                  (2u)
#define SignalBuffer_MODE                   (1u)
#define SignalBuffer_OA_COMP_TRIM_VALUE     (3u)
#define SignalBuffer_DEEPSLEEP_SUPPORT      (0u)


/***************************************
*    Variables with External Linkage
***************************************/

extern uint8  SignalBuffer_initVar;


/**************************************
*             Registers
**************************************/

#ifdef CYIPBLOCK_m0s8pass4b_VERSION
    #define SignalBuffer_CTB_CTRL_REG       (*(reg32 *) SignalBuffer_cy_psoc4_abuf__CTB_CTB_CTRL)
    #define SignalBuffer_CTB_CTRL_PTR       ( (reg32 *) SignalBuffer_cy_psoc4_abuf__CTB_CTB_CTRL)
#else
    #define SignalBuffer_CTB_CTRL_REG       (*(reg32 *) SignalBuffer_cy_psoc4_abuf__CTBM_CTB_CTRL)
    #define SignalBuffer_CTB_CTRL_PTR       ( (reg32 *) SignalBuffer_cy_psoc4_abuf__CTBM_CTB_CTRL)
#endif /* CYIPBLOCK_m0s8pass4b_VERSION */

#define SignalBuffer_OA_RES_CTRL_REG    (*(reg32 *) SignalBuffer_cy_psoc4_abuf__OA_RES_CTRL)
#define SignalBuffer_OA_RES_CTRL_PTR    ( (reg32 *) SignalBuffer_cy_psoc4_abuf__OA_RES_CTRL)
#define SignalBuffer_OA_COMP_TRIM_REG   (*(reg32 *) SignalBuffer_cy_psoc4_abuf__OA_COMP_TRIM)
#define SignalBuffer_OA_COMP_TRIM_PTR   ( (reg32 *) SignalBuffer_cy_psoc4_abuf__OA_COMP_TRIM)


/***************************************
*        Registers Constants
***************************************/

/* SignalBuffer_CTB_CTRL_REG */
#define SignalBuffer_CTB_CTRL_DEEPSLEEP_ON_SHIFT    (30u)   /* [30] Selects behavior CTB IP in the DeepSleep power mode */
#define SignalBuffer_CTB_CTRL_ENABLED_SHIFT         (31u)   /* [31] Enable of the CTB IP */


#define SignalBuffer_CTB_CTRL_DEEPSLEEP_ON          ((uint32) 0x01u << SignalBuffer_CTB_CTRL_DEEPSLEEP_ON_SHIFT)
#define SignalBuffer_CTB_CTRL_ENABLED               ((uint32) 0x01u << SignalBuffer_CTB_CTRL_ENABLED_SHIFT)


/* SignalBuffer_OA_RES_CTRL_REG */
#define SignalBuffer_OA_PWR_MODE_SHIFT          (0u)    /* [1:0]    Power level */
#define SignalBuffer_OA_DRIVE_STR_SEL_SHIFT     (2u)    /* [2]      Opamp output strenght select: 0 - 1x, 1 - 10x */
#define SignalBuffer_OA_COMP_EN_SHIFT           (4u)    /* [4]      CTB IP mode: 0 - Opamp, 1 - Comparator  */
#define SignalBuffer_OA_PUMP_EN_SHIFT           (11u)   /* [11]     Pump enable */


#define SignalBuffer_OA_PWR_MODE                ((uint32) 0x02u << SignalBuffer_OA_PWR_MODE_SHIFT)
#define SignalBuffer_OA_PWR_MODE_MASK           ((uint32) 0x03u << SignalBuffer_OA_PWR_MODE_SHIFT)
#define SignalBuffer_OA_DRIVE_STR_SEL_1X        ((uint32) 0x00u << SignalBuffer_OA_DRIVE_STR_SEL_SHIFT)
#define SignalBuffer_OA_DRIVE_STR_SEL_10X       ((uint32) 0x01u << SignalBuffer_OA_DRIVE_STR_SEL_SHIFT)
#define SignalBuffer_OA_DRIVE_STR_SEL_MASK      ((uint32) 0x01u << SignalBuffer_OA_DRIVE_STR_SEL_SHIFT)
#define SignalBuffer_OA_COMP_EN                 ((uint32) 0x00u << SignalBuffer_OA_COMP_EN_SHIFT)
#define SignalBuffer_OA_PUMP_EN                 ((uint32) 0x01u << SignalBuffer_OA_PUMP_EN_SHIFT)


/***************************************
*       Init Macros Definitions
***************************************/

#define SignalBuffer_GET_DEEPSLEEP_ON(deepSleep)    ((0u != (deepSleep)) ? (SignalBuffer_CTB_CTRL_DEEPSLEEP_ON) : (0u))
#define SignalBuffer_GET_OA_DRIVE_STR(current)      ((0u != (current)) ? (SignalBuffer_OA_DRIVE_STR_SEL_10X) : \
                                                                             (SignalBuffer_OA_DRIVE_STR_SEL_1X))
#define SignalBuffer_GET_OA_PWR_MODE(mode)          ((mode) & SignalBuffer_OA_PWR_MODE_MASK)
#define SignalBuffer_CHECK_PWR_MODE_OFF             (0u != (SignalBuffer_OA_RES_CTRL_REG & \
                                                                SignalBuffer_OA_PWR_MODE_MASK))

/* Returns true if component available in Deep Sleep power mode*/ 
#define SignalBuffer_CHECK_DEEPSLEEP_SUPPORT        (0u != SignalBuffer_DEEPSLEEP_SUPPORT) 

#define SignalBuffer_DEFAULT_CTB_CTRL (SignalBuffer_GET_DEEPSLEEP_ON(SignalBuffer_DEEPSLEEP_SUPPORT) | \
                                           SignalBuffer_CTB_CTRL_ENABLED)

#define SignalBuffer_DEFAULT_OA_RES_CTRL (SignalBuffer_OA_COMP_EN | \
                                              SignalBuffer_GET_OA_DRIVE_STR(SignalBuffer_OUTPUT_CURRENT))

#define SignalBuffer_DEFAULT_OA_COMP_TRIM_REG (SignalBuffer_OA_COMP_TRIM_VALUE)


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define SignalBuffer_LOWPOWER                   (SignalBuffer_LOW_POWER)
#define SignalBuffer_MEDPOWER                   (SignalBuffer_MED_POWER)
#define SignalBuffer_HIGHPOWER                  (SignalBuffer_HIGH_POWER)

/* PUMP ON/OFF defines */
#define SignalBuffer_PUMPON                     (SignalBuffer_PUMP_ON)
#define SignalBuffer_PUMPOFF                    (SignalBuffer_PUMP_OFF)

#define SignalBuffer_OA_CTRL                    (SignalBuffer_CTB_CTRL_REG)
#define SignalBuffer_OA_RES_CTRL                (SignalBuffer_OA_RES_CTRL_REG)

/* Bit Field  OA_CTRL */
#define SignalBuffer_OA_CTB_EN_SHIFT            (SignalBuffer_CTB_CTRL_ENABLED_SHIFT)
#define SignalBuffer_OA_PUMP_CTRL_SHIFT         (SignalBuffer_OA_PUMP_EN_SHIFT)
#define SignalBuffer_OA_PUMP_EN_MASK            (0x800u)
#define SignalBuffer_PUMP_PROTECT_MASK          (1u)


#endif    /* CY_OPAMP_SignalBuffer_H */


/* [] END OF FILE */
